
/***********************************************************/
/*                                                         */
/*    Source setup                                         */
/*                                                         */
/*    Compile and run with                                 */
/*                                                         */
/*    gcc 2d_source_main.c -lm -o 2d_source_main           */
/*                                                         */
/*    ./2d_source_main Nx Ny | ./plot_image.py -s Nx Ny    */
/*                                                         */
/***********************************************************/

                                 /* Michel Vallieres      */
                                 /* modifiied Fall 2012   */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>


#define MAX_x 700                        /* maximum dimensions   */
#define MAX_y 700

double drexel[MAX_y][MAX_x];             /* source arrays */
double old[MAX_y][MAX_x];
int sizex, sizey;



                                               /* display source phrase */
int see_image( char * image_name )
{
   int  ix, iy, iy_inverted;
   double  vmin, vmax;
   float  nodal[ MAX_x * MAX_y ];              /* linear array -> pointer access */
                                               /* min & max */
                                               /* create nodal variable */
   vmin = drexel[0][0];
   vmax = drexel[0][0];
   for ( iy=0 ; iy<sizey ; iy++ )
     {
      for ( ix=0 ; ix<sizex ; ix++ )
        {
           if ( drexel[iy][ix] < vmin ) vmin =  drexel[iy][ix];
           if ( drexel[iy][ix] > vmax ) vmax =  drexel[iy][ix];
	   iy_inverted = sizey - 1 - iy;
           nodal[ iy_inverted*sizex + ix ] = drexel[iy][ix];
        }

  }

   fprintf( stderr, " min & max: %f %f -- file name: %s\n", vmin, vmax,
                             image_name );

                                               /* write image data */
   for ( iy=0 ; iy<sizey ; iy++ )
     {
       for ( ix=0 ; ix<sizex ; ix++ )
	 {

           iy_inverted = sizey - 1 - iy ;
           printf( "%f\n", drexel[iy_inverted][ix] );
	 }
     }

}


                                               /* source smoothing    */
                                               /* by neighbor average */
int smooth_text()
{
   int    ix, iy, k, l, sum, level, level_max;

                                               /* level of smoothing */
   level_max = 4;
   for ( level=0 ; level < level_max ; level ++ )
     {
                                               /* smoothing */
     for ( iy=0 ; iy<sizey ; iy++ )
       {
         for ( ix=0 ; ix<sizex ; ix++ )
          {
             old[iy][ix] = drexel[iy][ix];
          }
       }
     for ( iy=2 ; iy<sizey-2 ; iy++ )
       {
        for ( ix=2 ; ix<sizex-2 ; ix++ )
          {
             sum = 0;
             for ( k=-1 ; k<2 ; k++ )
                for ( l=-1 ; l<2 ; l++ )
                   sum = sum + old[iy+k][ix+l];
             drexel[iy][ix] = sum;
          }
       }
     }
   fprintf( stderr, " smoothed %d times \n", level_max );
}


                                               /* form the letters */
void drexel_name()
{
   int    ix, iy, shift_text_x, shift_text_y, shiftx, shifty, 
       lx, ly, letterx, lettery, space;
   double x, y, a, b, scale_space, slope;

   for ( iy=0 ; iy<sizey ; iy++ )
     {
      for ( ix=0 ; ix<sizex ; ix++ )
        {
           drexel[iy][ix] = 0;
        }
     }

   lx = 3*sizex/4;
   ly = sizey/3;
   ly = ly - ly%4;
   shift_text_x = sizex/8;
   shift_text_y = sizey/3;
   scale_space = 0.9;
   letterx = lx / ( 6 + 5*scale_space );
   lettery = ly;
   space = scale_space * letterx;

                                         /* D */
   shiftx = shift_text_x;
   shifty = shift_text_y;
   for ( iy = shifty ; iy<shifty+lettery ; iy++)
                    drexel[iy][shiftx] = 1;
   a = letterx;
   b = lettery/2;
   for ( iy = shifty ; iy<=shifty+lettery ; iy++)
     {
        y = iy - shifty - lettery/2;
        x = a * sqrt( 1.0 - (y/b)*(y/b) );
        ix = shiftx + x + 0.1;
        drexel[iy][ix] = 1;
     }
                                         /* R */
   shiftx = shift_text_x + 1 * ( letterx + space );
   shifty = shift_text_y;
   for ( iy = shifty ; iy<=shifty+lettery ; iy++)
       drexel[iy][shiftx] = 1 ;
   a = letterx;
   b = lettery/4;
   for ( iy = shifty+lettery/2 ; iy<=shifty+lettery ; iy++)
     {
        y = iy - shifty - 3*lettery/4;
        x = a * sqrt( 1.0 - (y/b)*(y/b) );
        ix = shiftx + x + 0.1;
        drexel[iy-lettery/2][ix] = 1;
     }
   slope = 0.5 * (double)ly / (double)letterx;
   for ( iy = shifty+lettery/2 ; iy<=shifty+lettery ; iy++)
     {
        ix = (double)( iy - shifty -lettery/2 + slope*shiftx ) / slope + 0.1;
        drexel[iy][ix] = 1;
     }
                                         /* E */
   shiftx = shift_text_x + 2 * ( letterx + space );
   shifty = shift_text_y;
   for ( ix = shiftx ; ix<=shiftx+letterx ; ix++)
     {
       drexel[shifty][ix] = 1;
       drexel[shifty+ly/2][ix] = 1;
       drexel[shifty+ly][ix] = 1;
     }
   for ( iy = shifty ; iy<=shifty+lettery ; iy++)
       drexel[iy][shiftx] = 1 ;
                                         /* X */
   shiftx = shift_text_x + 3 * ( letterx + space );
   shifty = shift_text_y;
   slope = (double)ly/(double)letterx;
   for ( iy = shifty ; iy<=shifty+lettery ; iy++)
     {
        ix = (double)( iy - shifty + slope*shiftx ) / slope + 0.1;
        drexel[iy][ix] = 1;
        ix = (double)( -iy + shifty + ly + slope*shiftx ) / slope + 0.1;
        drexel[iy][ix] = 1;
     }
                                         /* E */
   shiftx = shift_text_x + 4 * ( letterx + space );
   shifty = shift_text_y;
   for ( ix = shiftx ; ix<=shiftx+letterx ; ix++)
     {
       drexel[shifty][ix] = 1;
       drexel[shifty+ly/2][ix] = 1;
       drexel[shifty+ly][ix] = 1;
     }
   for ( iy = shifty ; iy<=shifty+lettery ; iy++)
       drexel[iy][shiftx] = 1 ;
                                         /* L */
   shiftx = shift_text_x + 5 * ( letterx + space );
   shifty = shift_text_y;
   for ( ix = shiftx ; ix<=shiftx+letterx ; ix++)
     {
       drexel[shifty+lettery][ix] = 1;
     }
   for ( iy = shifty ; iy<=shifty+lettery ; iy++)
       drexel[iy][shiftx] = 1 ;
}


                                               /* phrase source */
void setup_source_amp( int raw, int smoothed )
{
   char name_text[] = "text.vtk";
   char name_smooth[] = "source.vtk";
                                               /* form the letters */
   drexel_name();
                                               /* display the letters */
   if ( raw == 1 )
	see_image( name_text );
                                               /* smooth the lettering */
   smooth_text();
                                               /* display the letters */
   if ( smoothed == 1 )
	see_image( name_smooth );
}


                                               /* source setup */
void setup_source(  int source )
{
   int    ix, iy, raw, smoothed;

   if ( source == 1 )
                                               /* Drexel source */
     {
        raw = 0;
        smoothed = 1;
        fprintf( stderr, "\n Source set-up\n" ); 
        setup_source_amp( raw, smoothed );
        fprintf( stderr, "\n" );
	                                       /* normalize source */
	                                       /* via amplitude */ 
        for ( iy=0 ; iy<sizey ; iy++ )
          {
            for ( ix=0 ; ix<sizex ; ix++ )
              {  
                drexel[iy][ix] =  2.5 *  drexel[iy][ix];
	      }
	  }
     }
                                               /* no source */
   else
     {
      for ( iy=0 ; iy<sizey ; iy++ )
        {
         for ( ix=0 ; ix<sizex ; ix++ )
	   {  
               drexel[iy][ix] = 0.0;
           }
        }
     }

}

